@@ -25,6 +25,10 @@
 
 #include <gnome.h>
 #include <pwd.h>
+#include <libgnomevfs/gnome-vfs.h>
+#include <libgnomevfs/gnome-vfs-mime-handlers.h>
+#include <libgnomevfs/gnome-vfs-mime-utils.h>
+#include <libgnomeui/gnome-thumbnail.h>
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
 #include <pwd.h>
@@ -47,6 +51,7 @@ typedef struct {
 
 	GdkScreen    	*screen;
 	GtkIconTheme 	*theme;
+	GnomeThumbnailFactory *thumbs;
 
 	EContactAddress *addr1;
 	EContactAddress *addr2;
@@ -262,8 +267,13 @@ get_user_login (void)
 	struct passwd pwd, *err;
 
 	int i;
-	i = getpwuid_r(getuid(), &pwd, buf, sizeof(buf), &err);
-	return ((i == 0) && (err == &pwd)) ? g_strdup(pwd.pw_name) : NULL;
+#if __sun
+	i = getpwuid_r (getuid (), &pwd, buf, sizeof (buf));
+	return (i != 0) ? g_strdup (pwd.pw_name) : NULL;
+#else
+	i = getpwuid_r (getuid (), &pwd, buf, sizeof (buf), &err);
+	return ((i == 0) && (err == &pwd)) ? g_strdup (pwd.pw_name) : NULL;
+#endif
 }
 
 /*
@@ -540,6 +550,42 @@ eab_create_image_chooser_widget (gchar *name,
 	return w;
 }
 
+static void 
+about_me_update_preview (GtkFileChooser *chooser,
+			 GnomeAboutMe   *me) 
+{
+	gchar *uri;
+
+	uri = gtk_file_chooser_get_preview_uri (chooser);
+	
+	if (uri) {
+		GtkWidget *image;
+		GdkPixbuf *pixbuf;
+		gchar *mime_type;
+
+		if (!me->thumbs)
+			me->thumbs = gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_NORMAL);
+
+		
+		mime_type = gnome_vfs_get_mime_type (uri);
+		pixbuf = gnome_thumbnail_factory_generate_thumbnail (me->thumbs,
+								     uri,
+								     mime_type);
+		image = gtk_file_chooser_get_preview_widget (chooser);
+		
+		if(pixbuf != NULL) {
+			gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
+			g_object_unref (pixbuf);
+		} else {
+			gtk_image_set_from_stock (GTK_IMAGE (image),
+						  "gtk-dialog-question",
+						  GTK_ICON_SIZE_DIALOG);
+		}
+		g_free (mime_type);
+	}
+	gtk_file_chooser_set_preview_widget_active (chooser, TRUE);
+}
+
 static void
 about_me_image_clicked_cb (GtkWidget *button, GnomeAboutMe *me)
 {
@@ -547,6 +593,7 @@ about_me_image_clicked_cb (GtkWidget *button, GnomeAboutMe *me)
 	gint response;
 	GtkWidget *image_chooser;
 	GladeXML  *dialog;
+	GtkWidget *image;
 	
 	dialog = me->dialog;
 	image_chooser = WID ("image-chooser");
@@ -561,6 +608,19 @@ about_me_image_clicked_cb (GtkWidget *button, GnomeAboutMe *me)
 	gtk_dialog_set_default_response (GTK_DIALOG (chooser_dialog), GTK_RESPONSE_ACCEPT);
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser_dialog), g_get_home_dir ());
 
+	gtk_file_chooser_set_use_preview_label (GTK_FILE_CHOOSER (chooser_dialog),
+						FALSE);
+	
+	image = gtk_image_new ();
+	gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (chooser_dialog),
+					     image);
+	gtk_widget_set_size_request (image, 128, -1);
+	
+	gtk_widget_show (image);
+
+	g_signal_connect (chooser_dialog, "update-preview",
+			  G_CALLBACK (about_me_update_preview), me);
+
 	response=gtk_dialog_run (GTK_DIALOG (chooser_dialog));
 
 	if (response == GTK_RESPONSE_ACCEPT) {
@@ -652,7 +712,7 @@ about_me_passwd_clicked_cb (GtkWidget *button, GnomeAboutMe *me)
 	gnome_about_me_password (GTK_WINDOW (WID ("about-me-dialog")));
 }
 
-static void
+static gint
 about_me_setup_dialog (void)
 {
 	GtkWidget    *widget;
@@ -702,6 +762,12 @@ about_me_setup_dialog (void)
 
 	/* Get the self contact */
 	if (!e_book_get_self (&me->contact, &me->book, &error)) {
+		if (error->code == E_BOOK_ERROR_PROTOCOL_NOT_SUPPORTED) {
+			about_me_error (NULL, _("There was an error while trying to get the addressbook information\n" \
+						"Evolution Data Server can't handle the protocol"));
+			return -1;
+		}	
+
 		me->create_self = TRUE;
 		
 		me->contact = e_contact_new ();
@@ -731,7 +797,7 @@ about_me_setup_dialog (void)
 	if (pwent == NULL) {
 		about_me_error (GTK_WINDOW (WID ("about-me-dialog")), 
 				_("Unknown login ID, the user database might be corrupted"));
-		return ;
+		return -1;
 	}
 	gchar **tok;
 	tok = g_strsplit (pwent->pw_gecos, ",", 0);
@@ -783,6 +849,8 @@ about_me_setup_dialog (void)
 	about_me_load_info (me);
 
 	gtk_widget_show_all (main_dialog);
+
+	return 0;
 }
 
 int
@@ -797,8 +865,11 @@ main (int argc, char **argv)
 		GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
 		NULL);
 
-	about_me_setup_dialog ();
-	gtk_main ();
+	if (about_me_setup_dialog () == -1) {
+		return -1;
+	} else {
+		gtk_main ();
+	}
 
 	return 0;
 }
