@@ -1,7 +1,7 @@
 /*
  *  Authors: Rodney Dawes <dobey@ximian.com>
  *
- *  Copyright 2003-2005 Novell, Inc. (www.novell.com)
+ *  Copyright 2003-2006 Novell, Inc. (www.novell.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of version 2 of the GNU General Public License
@@ -56,7 +56,11 @@ GnomeWPItem * gnome_wp_item_new (const gchar * filename,
   if (item->fileinfo != NULL &&
       !strncmp (item->fileinfo->mime_type, "image/", strlen ("image/"))) {
     if (item->name == NULL) {
-      item->name = g_strdup (item->fileinfo->name);
+      if (g_utf8_validate (item->fileinfo->name, -1, NULL))
+	item->name = g_strdup (item->fileinfo->name);
+      else
+	item->name = g_filename_to_utf8 (item->fileinfo->name, -1, NULL,
+					 NULL, NULL);
     }
     item->options = gconf_client_get_string (client, WP_OPTIONS_KEY, NULL);
 
@@ -270,6 +274,15 @@ GdkPixbuf * gnome_wp_item_get_thumbnail (GnomeWPItem * item,
 	tw = bw;
       }
       scaled = gnome_wp_pixbuf_center (pixbuf, bgpixbuf, tw, th);
+    } else if (!strcmp (item->options, "zoom")) {
+      if ((gdouble) ph * (gdouble) bw < (gdouble) pw * (gdouble) bh) {
+	tw = 0.5 + (gdouble) pw * (gdouble) bh / (gdouble) ph; 
+	th = bh;
+      } else {
+	th = 0.5 + (gdouble) ph * (gdouble) bw / (gdouble) pw;
+	tw = bw;
+      }
+      scaled = gnome_wp_pixbuf_center (pixbuf, bgpixbuf, tw, th);
     }
   }
 
